@@ -32,6 +32,7 @@
 #include <libqmi-glib.h>
 
 #include "qmicli.h"
+#include "qmicli-helpers.h"
 
 #define PROGRAM_NAME    "qmicli"
 #define PROGRAM_VERSION PACKAGE_VERSION
@@ -50,6 +51,7 @@ static gboolean get_service_version_info_flag;
 static gchar *device_set_instance_id_str;
 static gboolean device_open_version_info_flag;
 static gboolean device_open_sync_flag;
+static gchar *device_open_net_str;
 static gboolean device_open_proxy_flag;
 static gchar *client_cid_str;
 static gboolean client_no_release_cid_flag;
@@ -82,6 +84,10 @@ static GOptionEntry main_entries[] = {
       "Request to use the 'qmi-proxy' proxy",
       NULL
     },
+    { "device-open-net", 0, 0, G_OPTION_ARG_STRING, &device_open_net_str,
+      "Open device with specific link protocol and QoS flags",
+      "[net-802-3|net-raw-ip|net-qos-header|net-no-qos-header]"
+    },
     { "client-cid", 0, 0, G_OPTION_ARG_STRING, &client_cid_str,
       "Use the given CID, don't allocate a new one",
       "[CID]"
@@ -300,6 +306,9 @@ allocate_client_ready (QmiDevice *dev,
     case QMI_SERVICE_UIM:
         qmicli_uim_run (dev, QMI_CLIENT_UIM (client), cancellable);
         return;
+    case QMI_SERVICE_WDA:
+        qmicli_wda_run (dev, QMI_CLIENT_WDA (client), cancellable);
+        return;
     default:
         g_assert_not_reached ();
     }
@@ -481,6 +490,9 @@ device_new_ready (GObject *unused,
         open_flags |= QMI_DEVICE_OPEN_FLAGS_SYNC;
     if (device_open_proxy_flag)
         open_flags |= QMI_DEVICE_OPEN_FLAGS_PROXY;
+    if (device_open_net_str)
+        if (!qmicli_read_net_open_flags_from_string (device_open_net_str, &open_flags))
+            exit (EXIT_FAILURE);
 
     /* Open the device */
     qmi_device_open (device,
@@ -534,6 +546,12 @@ parse_actions (void)
         actions_enabled++;
     }
 
+    /* WDA options? */
+    if (qmicli_wda_options_enabled ()) {
+        service = QMI_SERVICE_WDA;
+        actions_enabled++;
+    }
+
     /* Cannot mix actions from different services */
     if (actions_enabled > 1) {
         g_printerr ("error: cannot execute multiple actions of different services\n");
@@ -571,6 +589,8 @@ int main (int argc, char **argv)
 	                            qmicli_pbm_get_option_group ());
 	g_option_context_add_group (context,
 	                            qmicli_uim_get_option_group ());
+	g_option_context_add_group (context,
+	                            qmicli_wda_get_option_group ());
     g_option_context_add_main_entries (context, main_entries, NULL);
     if (!g_option_context_parse (context, &argc, &argv, &error)) {
         g_printerr ("error: %s\n",
